/* arithparse.mly: ocaml yacc sourc file defining a parser. This
   version produces an expression tree rather than direclty computing
   its answer. */

/* first section defines token types used by parser using % directives */
%token <int> INT
%token PLUS MINUS TIMES SLASH
%token OPAREN CPAREN
%token EOL

%type <Arithexpr.expr> main                  /* main produces an expr tree in this version */
%start main                                  /* entry production for parser */

/* end first section */
%%
/* second section which defines grammar productions */

main:                                         /* initial production */
  | plusminus EOL             { $1 }          /* $1 refers to value of first production */
;
plusminus:                                    /* addition and subtraction */
  | muldiv                    { $1 }          /* could be just mul/div */
  | plusminus PLUS  muldiv    { Add($1,$3) }  /* or an addition */
  | plusminus MINUS muldiv    { Sub($1,$3) }  /* or a subtraction */
;
muldiv:                                       /* multiplication and division */
  | ident                     { $1 }          /* could be just and identifier */
  | muldiv TIMES ident        { Mul($1,$3) }  /* or a multiplication */
  | muldiv SLASH ident        { Div($1,$3) }  /* or a division */
;
ident:                                        /* identifier */
  | INT                       { Const($1) }   /* integer constant */
  | OPAREN plusminus CPAREN   { $2 }          /* opening parenthesis */
;

/* End second section for grammar productions  */

